{GradText - Harm: harmdog@ne.uswest.net
 Version 1.0 - 2/98 - Initial release to the public}

unit GradText;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls;

type
  TGradientStyle = (gsHorizontal, gsVertical, gsRectangle,
                    gsVertCenter, gsHorizCenter);

  TJustification = (jLeft, jCenter, jRight);

  TGradText = class(TGraphicControl)
  private
    FBeginClr : TColor;    //Start color for gradient
    FEndClr : TColor;     //End color for gradient
    FGradientStyle : TGradientStyle;
    FLines : Tstrings;    //Variable for getting the text
    FCaption : string;    //Variable for use with DrawText API
    FJustify : TJustification;
    bmB : TBitmap;        //Internal BMP's
    bmT : TBitmap;
    procedure SetCaption(Value : string);
    procedure Paint; override;
    procedure SetBeginClr(Value : TColor);  //Set start color of gradient
    procedure SetEndClr(Value : TColor);    //Set end color of gradient
    procedure SetGradient(Value : TGradientStyle);  //Choose gradient fill style
    procedure SetLines(Value : Tstrings);    //Update the text
    procedure SetJustify(Value : TJustification);
    procedure DoText;   //Draw the text
    procedure DoHorizontal(fr, fg, fb, dr, dg, db : Integer); //These all draw
    procedure DoVertical(fr, fg, fb, dr, dg, db : Integer); // the gradients.
    procedure DoRectangle(fr, fg, fb, dr, dg, db : Integer);
    procedure DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
    procedure DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
  protected
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property Caption : string read FCaption write SetCaption; //Not published
  published
    property BeginColor : TColor read FBeginClr
       write SetBeginClr default clYellow;
    property EndColor : TColor read FEndClr
       write SetEndClr default clRed;
    property GradientStyle : TGradientStyle read FGradientStyle
       write SetGradient default gsVertical;
    property Lines : TStrings read FLines
       write SetLines;
    property Justification : TJustification read FJustify
       write SetJustify default jCenter;
    property Font;
    property Visible;
  end;

procedure Register;

implementation

constructor TGradText.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csNoStdEvents];  //Make us faster (I guess)
  Font.Size := 18;                 //Make the default fairly large
  Font.Name := 'Times New Roman';  //A font every Win95 user should have
  Font.Style := [fsBold,fsItalic]; 
  FBeginClr := clYellow;
  FEndClr := clRed;
  FGradientStyle := gsVertical;
  FJustify := jCenter;
  FLines := TStringList.Create;   //Even though FLines is a TStrings type,
                                  // I cast it as a TStringList to get
                                  // the default property editor to work.
  with TStringList ( FLines ) do begin
    Sorted := FALSE;    //Set a couple of properties we care about
    Add('GradText');    //This is the default text.  You'll notice that
  end;                  // it doesn't use the Name+ComponentIndex scheme.
  Height := 25;   //I put some values here, but they get adjusted
  Width := 75;    // depending on Font size. Prob'ly don't need.
  bmB := TBitmap.Create;          //Create the Background BMP (Gradient)
  bmT := TBitmap.Create;          //Create the Text BMP (Mask)
  bmT.Canvas.Brush.Style := bsSolid; //Prob'ly the default
  bmT.Canvas.Brush.Color := clWhite; //Set 'Mask' colors for the text canvas
  bmT.Canvas.Font.Color := clBlack;
  bmT.Transparent := TRUE;
  bmT.TransparentColor := clBlack;   //Transparent color same as font.
  bmB.Transparent := TRUE;
  bmB.TransparentColor := clWhite;
  Canvas.Brush.Style := bsClear;  //This makes our control transparent.
end;

procedure TGradText.Paint;
var
  FromR, FromG, FromB : Integer;
  DiffR, DiffG, DiffB : Integer;
  r : TRect;  //This rect is used for sizing the control
  a : integer;  //Just a loop-counter
  J : integer;  //For Justification
begin
  J := 0;  //Initialized so I don't get warnings at compile time
  FCaption := '';  //Always re-build the string at paint time
  //DrawText API requires a ptr to a string, so the Lines must be
  // converted to a single entity.  Hence, FCaption and this loop:
  for a := 0 to FLines.Count - 2 do
    FCaption := FCaption + FLines.Strings[a] + #13;
  FCaption := FCaption + FLines.Strings[Flines.Count-1];
  bmB.Canvas.Font := Font;  //Synch bmp fonts with the control font property
  bmT.Canvas.Font := Font;
  case FJustify of
    jLeft   : J := DT_LEFT;
    JCenter : J := DT_CENTER;
    jRight  : J := DT_RIGHT;
  end;
  r := RECT(0,0,Width,Height);  //Initialize the rect to something.
  //The following code does not draw any text.  The DT_CALCRECT parameter
  // gets the dimensions required to hold the text specified and stores
  // it in variable r.  The dimensions are used for sizing the bmp's so
  // the gradient matches the text rect.  See help for other parameters.
  DrawText(bmT.Canvas.Handle,PChar(FCaption),-1,r,
      DT_CALCRECT + J + DT_WORDBREAK + DT_NOPREFIX);
  bmB.Width := r.Right;          //Set BMP dimensions to match text rect
  bmB.Height := r.Bottom;
  bmT.Width := r.Right;
  bmT.Height := r.Bottom;
  Width := bmT.Width;        //Set Control dimensions to match text rect
  Height := bmT.Height;
  FromR := FBeginClr and $000000ff;  //Strip out separate RGB values
  FromG := (FBeginClr shr 8) and $000000ff;
  FromB := (FBeginClr shr 16) and $000000ff;
  DiffR := (FEndClr and $000000ff) - FromR;   //Find the difference
  DiffG := ((FEndClr shr 8) and $000000ff) - FromG;
  DiffB := ((FEndClr shr 16) and $000000ff) - FromB;
  case FGradientStyle of    //Go draw gradient
    gsHorizontal  : DoHorizontal(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertical    : DoVertical(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsRectangle   : DoRectangle(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsVertCenter  : DoVertCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
    gsHorizCenter : DoHorizCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  end;
  DoText;   //Gradient is drawn, now go draw the text
  bmB.Canvas.Draw(0,0,bmT);  //Overlay the gradient with our text mask
  Canvas.Draw(0,0,bmB);  //Overlay our control canvas.
end;

procedure TGradText.DoText;
var
  r : TRect;
  J : integer;
begin
  J := 0;  //I initialize this so I don't get warnings at compile time.
  bmT.Canvas.FillRect(RECT(0,0,bmT.Width,bmT.Height));  //Erase old stuff
  r := RECT(0,0,bmT.Width,bmT.Height);  //Set rect dimensions
  case FJustify of
    jLeft   : J := DT_LEFT;
    JCenter : J := DT_CENTER;
    jRight  : J := DT_RIGHT;
  end;
  DrawText(bmT.Canvas.Handle,PChar(FCaption),-1,r,  //Draw text
      J + DT_WORDBREAK + DT_NOPREFIX);
end;

procedure TGradText.SetCaption(Value : string);
begin
  if FCaption <> Value then begin
    FCaption := Value;
    FLines.Text := Value;
    Invalidate;
  end;
end;

procedure TGradText.SetLines(Value : TStrings);
begin                            //I decided to prevent a blank control by
  if Value.Count = 0 then exit;  // exiting this procedure if 'Lines' is blank.
  FLines.Assign(Value);          // It retains the old text, but allows the
  Invalidate;                    // 'Lines' property to have 0 lines.
end;                             // Seems odd, but it works for me.

procedure TGradText.SetBeginClr(Value : TColor);
begin
  if FBeginClr <> Value then begin
    FBeginClr := Value;
    Invalidate;
  end;
end;

procedure TGradText.SetEndClr(Value : TColor);
begin
  if FEndClr <> Value then begin
    FEndClr := Value;
    Invalidate;
  end;
end;

procedure TGradText.SetGradient(Value : TGradientStyle);
begin
  if FGradientStyle <> Value then begin
    FGradientStyle := Value;
    Invalidate;
  end;
end;

procedure TGradText.SetJustify(Value : TJustification);
begin
  if FJustify <> Value then begin
    FJustify := Value;
    Invalidate;
  end;
end;

procedure TGradText.DoHorizontal(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Top:= 0;                //Set rectangle top
  ColorRect.Bottom := Height;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Left:= MulDiv (I, Width, 256);    //Find left for this color
    ColorRect.Right:= MulDiv (I + 1, Width, 256);   //Find Right
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bmB.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bmB.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradText.DoVertical(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Left:= 0;                //Set rectangle left&right
  ColorRect.Right:= Width;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Top:= MulDiv (I, Height, 256);    //Find top for this color
    ColorRect.Bottom:= MulDiv (I + 1, Height, 256);   //Find Bottom
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bmB.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bmB.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradText.DoRectangle(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph : Real;
  x1,y1,x2,y2 : Real;
begin
  bmB.Canvas.Pen.Style := psClear;
  bmB.Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := bmB.Width+2;
  y1 := 0;
  y2 := bmB.Height+2;
  Pw := (bmB.Width / 2) / 255;
  Ph := (bmB.Height / 2) / 255;
  for I := 0 to 255 do begin         //Make rectangles of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bmB.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bmB.Canvas.FillRect(Rect(Trunc(x1),Trunc(y1),Trunc(x2),Trunc(y2)));
  end;
  bmB.Canvas.Pen.Style := psSolid;
end;

procedure TGradText.DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bmB.Height Div 2;
  ColorRect.Left := 0;
  ColorRect.Right := bmB.Width;
  for I := 0 to Haf do begin
    ColorRect.Top := MulDiv (I, Haf, Haf);
    ColorRect.Bottom := MulDiv (I + 1, Haf, Haf);
    R := fr + MulDiv(I, dr, Haf);
    G := fg + MulDiv(I, dg, Haf);
    B := fb + MulDiv(I, db, Haf);
    bmB.Canvas.Brush.Color := RGB(R, G, B);
    bmB.Canvas.FillRect(ColorRect);
    ColorRect.Top := bmB.Height - (MulDiv (I, Haf, Haf));
    ColorRect.Bottom := bmB.Height - (MulDiv (I + 1, Haf, Haf));
    bmB.Canvas.FillRect(ColorRect);
  end;
end;

procedure TGradText.DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bmB.Width Div 2;
  ColorRect.Top := 0;
  ColorRect.Bottom := bmB.Height;
  for I := 0 to Haf do begin
    ColorRect.Left := MulDiv (I, Haf, Haf);
    ColorRect.Right := MulDiv (I + 1, Haf, Haf);
    R := fr + MulDiv(I, dr, Haf);
    G := fg + MulDiv(I, dg, Haf);
    B := fb + MulDiv(I, db, Haf);
    bmB.Canvas.Brush.Color := RGB(R, G, B);
    bmB.Canvas.FillRect(ColorRect);
    ColorRect.Left := bmB.Width - (MulDiv (I, Haf, Haf));
    ColorRect.Right := bmB.Width - (MulDiv (I + 1, Haf, Haf));
    bmB.Canvas.FillRect(ColorRect);
  end;
end;

destructor TGradText.Destroy;   //Free everything created dynamically
begin
  bmB.Free;
  bmT.Free;
  FLines.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TGradText]);
end;

end.
